package gov.anl.aps.dm.portal.controllers;

import gov.anl.aps.dm.portal.exceptions.DmPortalException;
import gov.anl.aps.dm.portal.exceptions.MissingProperty;
import gov.anl.aps.dm.portal.exceptions.ObjectAlreadyExists;
import gov.anl.aps.dm.portal.model.entities.AllowedPolicyValue;
import gov.anl.aps.dm.portal.model.beans.AllowedPolicyValueFacade;
import gov.anl.aps.dm.portal.model.entities.PolicyProperty;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.ejb.EJB;
import javax.inject.Named;
import javax.enterprise.context.SessionScoped;
import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;

import org.apache.log4j.Logger;

@Named("allowedPolicyValueController")
@SessionScoped
public class AllowedPolicyValueController extends CrudEntityController<AllowedPolicyValue, AllowedPolicyValueFacade>
{
    private static final Logger logger = Logger.getLogger(AllowedPolicyValueController.class.getName());
    
    class ValueCopy {
        private final String value;
        private final String description;

        ValueCopy(String value, String description) {
            this.value = value;
            this.description = description;
        }
        
        boolean isUpdated(AllowedPolicyValue editedAllowedPolicyValue) {
            return (!editedAllowedPolicyValue.getPolicyValue().equals(value) || !editedAllowedPolicyValue.getDescription().equals(description));
        }
        
        String getValue() {
            return value;
        }
        
        String getDescription() {
            return description;
        }
    }

    @EJB
    private AllowedPolicyValueFacade allowedPolicyValueFacade;
    
    private final Map<AllowedPolicyValue, ValueCopy> editMap = new HashMap<>();

    class AllowedPolicyValueInfoTable extends DataTableController<AllowedPolicyValue> {

        @Override
        public String getEntityName() {
            return "AllowedPolicyValue";
        }

        @Override
        public List<AllowedPolicyValue> findAll() {
            return allowedPolicyValueFacade.findByPolicyPropertyId(getPolicyProperty().getId());
        }

        @Override
        public String getTableName() {
            return "allowedPolicyValueInfoTable";
        }
    }

    private AllowedPolicyValueInfoTable allowedPolicyValueInfoTable = new AllowedPolicyValueInfoTable();
    private PolicyProperty policyProperty;
    private int rows = 25;

    public AllowedPolicyValueController() {
    }

    public AllowedPolicyValueInfoTable getAllowedPolicyValueInfoTable() {
        return allowedPolicyValueInfoTable;
    }

    public void setAllowedPolicyValueInfoTable(AllowedPolicyValueInfoTable allowedPolicyValueInfoTable) {
        this.allowedPolicyValueInfoTable = allowedPolicyValueInfoTable;
    }

    public PolicyProperty getPolicyProperty() {
        return policyProperty;
    }

    public void setPolicyProperty(PolicyProperty policyProperty) {
        clear();
        this.policyProperty = policyProperty;
    }

    public int getRows() {
        return rows;
    }

    public void setRows(int rows) {
        this.rows = rows;
    }

    @Override
    protected AllowedPolicyValueFacade getFacade() {
        return allowedPolicyValueFacade;
    }

    @Override
    public DataModel createListDataModel() {
        List<AllowedPolicyValue> allowedValueList = allowedPolicyValueFacade.findByPolicyPropertyId(policyProperty.getId());        
        for (AllowedPolicyValue allowedPolicyValue : allowedValueList) {
            editMap.put(allowedPolicyValue, new ValueCopy(allowedPolicyValue.getPolicyValue(), allowedPolicyValue.getDescription()));
        }
        return new ListDataModel((List) allowedValueList);
    }

    @Override
    public List<AllowedPolicyValue> getAvailableItems() {
        List<AllowedPolicyValue> allowedValueList = allowedPolicyValueFacade.findByPolicyPropertyId(policyProperty.getId());
        return (allowedValueList);
    }

    @Override
    protected AllowedPolicyValue createEntityInstance() {
        return new AllowedPolicyValue();
    }

    @Override
    public String getEntityTypeName() {
        return "AllowedPolicyValue";
    }

    @Override
    public String getCurrentEntityInstanceName() {
        if (getCurrent() != null) {
            return getCurrent().getPolicyValue();
        }
        return "";
    }

    @Override
    public String prepareEdit(AllowedPolicyValue allowedPolicyValue) {
        super.prepareEdit(allowedPolicyValue);
        return "/views/allowedPolicyValue/edit?faces-redirect=true";
    }

    @Override
    public String prepareView(AllowedPolicyValue allowedPolicyValue) {
        current = allowedPolicyValue;
        super.prepareView(allowedPolicyValue);
        return "/views/allowedPolicyValue/view?faces-redirect=true";
    }

    @Override
    public String prepareCreate() {
        super.prepareCreate();
        return "/views/allowedPolicyValue/create?faces-redirect=true";
    }

    @Override
    public String create() { 
        AllowedPolicyValue newValue = current;
        if (super.create() == null) {
            return null;
        }
        policyProperty.getAllowedPolicyValueList().add(newValue);
        allowedPolicyValueInfoTable.resetList();
        return null;
    }
    
     @Override
    public String destroy() {
        AllowedPolicyValue propertyValue = current;
        super.destroy();
        policyProperty.getAllowedPolicyValueList().remove(propertyValue);
        clear();
        return null;
    }

    @Override
    public void prepareEntityInsert(AllowedPolicyValue allowedPolicyValue) throws ObjectAlreadyExists, MissingProperty {
        current.setPolicyProperty(policyProperty);
         if ((allowedPolicyValue.getPolicyValue() == null) || (allowedPolicyValue.getPolicyValue().length() == 0)) {
            throw new MissingProperty("Policy Value is missing.");
        }
        List<AllowedPolicyValue> propertyAllowedPolicyValueList = allowedPolicyValueFacade.findByPolicyPropertyId(policyProperty.getId());
        for (AllowedPolicyValue policyValue : propertyAllowedPolicyValueList) {
            if ((allowedPolicyValue.getPolicyValue().equals(policyValue.getPolicyValue()))) {
                throw new ObjectAlreadyExists("allowed policy value " + allowedPolicyValue.getPolicyValue() + " already exists for this property.");
            }
        }
  }

    @Override
    public void prepareEntityUpdate(AllowedPolicyValue allowedPolicyValue) throws DmPortalException {
        if ((allowedPolicyValue.getPolicyValue() == null) || (allowedPolicyValue.getPolicyValue().length() == 0)) {
            throw new MissingProperty("Policy Value is missing.");
        }
        ValueCopy tempValueCopy = editMap.get(allowedPolicyValue);
        editMap.remove(allowedPolicyValue);
        Collection<ValueCopy> propertyAllowedPolicyValueList = editMap.values();
        for (ValueCopy valueCopy : propertyAllowedPolicyValueList) {
            if ((valueCopy.getValue().equals(allowedPolicyValue.getPolicyValue()))) {
                throw new ObjectAlreadyExists("allowed policy value " + allowedPolicyValue.getPolicyValue() + " already exists for this property.");
            }
        }
        editMap.put(allowedPolicyValue, tempValueCopy);
    }

    @Override
    public String prepareList() {
        logger.debug("Preparing list");
        resetListDataModel();
        return "/views/policyProperty/view?faces-redirect=true";
    }
    
    @Override
    protected String getObjectAlreadyExistMessage(AllowedPolicyValue entity) {
        return null;
    }
    
    @Override
    public void clear() {
        allowedPolicyValueInfoTable.resetList();
        resetList();
        editMap.clear();
    }
    
    @Override
    public String update() {
        Iterator<AllowedPolicyValue> iterator = getListDataModel().iterator();
        while (iterator.hasNext()) {
            AllowedPolicyValue editedAllowedPolicyValue = iterator.next();
            ValueCopy valueCopy = editMap.get(editedAllowedPolicyValue);
            if ((valueCopy != null) && (valueCopy.isUpdated(editedAllowedPolicyValue))) {
                current = editedAllowedPolicyValue;
                super.update();
            }            
        }
        clear();
        return null;
    }

}
    