package gov.anl.aps.dm.portal.controllers;

import gov.anl.aps.dm.portal.model.entities.CloneableEntity;
import java.io.Serializable;
import java.util.List;
import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;
import org.apache.log4j.Logger;
import org.primefaces.component.datatable.DataTable;

public abstract class DataTableController<EntityType extends CloneableEntity> implements Serializable
{

    private static final Logger logger = Logger.getLogger(DataTableController.class.getName());


    protected EntityType currentObject = null;

    private DataModel listDataModel = null;
    private DataTable listDataTable = null;
    private List<EntityType> filteredObjectList = null;
    private List<EntityType> selectedObjectList = null;

        public abstract List<EntityType> findAll();
    
    public abstract String getTableName();

    public abstract String getEntityName();

    public EntityType getCurrentObject() {
        return currentObject;
    }

    public void setCurrentObject(EntityType current) {
        this.currentObject = current;
    }

    public DataModel createListDataModel() {
        return new ListDataModel(findAll());
    }

    public DataModel getListDataModel() {
        if (listDataModel == null) {
            listDataModel = createListDataModel();
        }
        return listDataModel;
    }

    public void resetList() {
        logger.debug("Resetting list"+" "+getTableName());
        listDataModel = null;
        listDataTable = null;
        filteredObjectList = null;
        selectedObjectList = null;
        currentObject = null;
    }

   public DataTable getListDataTable() {
        if (listDataTable == null) {
            logger.debug("Recreating data table"+" "+getTableName());
            listDataTable = new DataTable();
        }
        return listDataTable;
    }

    public void setListDataTable(DataTable listDataTable) {
        this.listDataTable = listDataTable;
    }

    public List<EntityType> getFilteredObjectList() {
        return filteredObjectList;
    }

     public void setFilteredObjectList(List<EntityType> filteredObjectList) {
        this.filteredObjectList = filteredObjectList;
    }

    public List<EntityType> getFilteredItems() {
        return filteredObjectList;
    }

    public void setFilteredItems(List<EntityType> filteredItems) {
        this.filteredObjectList = filteredItems;
    }
    
    public void resetFilterList() {
        filteredObjectList = null;
    }
    
    public List<EntityType> getSelectedObjectList() {
        return selectedObjectList;
    }
    
    public void setSelectedObjectList(List<EntityType> selectedObjectList) {
        this.selectedObjectList = selectedObjectList;
    }
    
}
