
package gov.anl.aps.dm.portal.controllers;

import gov.anl.aps.dm.api.ExperimentDsApi;
import gov.anl.aps.dm.common.exceptions.DmException;
import gov.anl.aps.dm.portal.constants.RoleTypeName;
import gov.anl.aps.dm.portal.exceptions.DmPortalException;
import gov.anl.aps.dm.portal.exceptions.ObjectAlreadyExists;
import gov.anl.aps.dm.portal.exceptions.InvalidDate;
import gov.anl.aps.dm.portal.exceptions.MissingProperty;
import gov.anl.aps.dm.portal.model.entities.Experiment;
import gov.anl.aps.dm.portal.model.beans.ExperimentFacade;
import gov.anl.aps.dm.portal.model.beans.ExperimentPolicyFacade;
import gov.anl.aps.dm.portal.model.beans.ExperimentPolicyPropertyValueFacade;
import gov.anl.aps.dm.portal.model.beans.PolicyTypeFacade;
import gov.anl.aps.dm.portal.model.beans.RoleTypeFacade;
import gov.anl.aps.dm.portal.model.beans.UserExperimentRoleFacade;
import gov.anl.aps.dm.portal.model.beans.UserInfoFacade;
import gov.anl.aps.dm.portal.model.entities.CloneableEntity;
import gov.anl.aps.dm.portal.model.entities.ExperimentPolicy;
import gov.anl.aps.dm.portal.model.entities.ExperimentPolicyPropertyValue;
import gov.anl.aps.dm.portal.model.entities.PolicyProperty;
import gov.anl.aps.dm.portal.model.entities.PolicyType;
import gov.anl.aps.dm.portal.model.entities.RoleType;
import gov.anl.aps.dm.portal.model.entities.UserExperimentRole;
import gov.anl.aps.dm.portal.model.entities.UserInfo;
import gov.anl.aps.dm.common.utilities.CollectionUtility;
import gov.anl.aps.dm.portal.utilities.DmApiFactory;
import gov.anl.aps.dm.portal.utilities.SessionUtility;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.ejb.EJB;
import javax.inject.Named;
import javax.enterprise.context.SessionScoped;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.FacesConverter;
import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;
import javax.faces.model.SelectItem;
import org.apache.log4j.Logger;

@Named("experimentController")
@SessionScoped
public class ExperimentController extends CrudEntityController<Experiment, ExperimentFacade>
{
    private static final Logger logger = Logger.getLogger(ExperimentController.class.getName());

    @EJB
    private ExperimentFacade experimentFacade;

    @EJB
    private UserInfoFacade userInfoFacade;

    @EJB
    private RoleTypeFacade roleTypeFacade;
        
    @EJB
    private UserExperimentRoleFacade userExperimentRoleFacade;
    
    @EJB
    private PolicyTypeFacade policyTypeFacade;
    
    @EJB
    private ExperimentPolicyFacade experimentPolicyFacade;
    
    @EJB
    private ExperimentPolicyPropertyValueFacade experimentPolicyPropertyValueFacade;
            
    public class ExperimentUser extends CloneableEntity {

        String name;
        int experimentId;
        private final UserInfo user;
        private final boolean[] userRoles;

        public ExperimentUser(int experimentId, UserInfo user) {
            userRoles = new boolean[maxExperimentRoleTypeId + 1];
            this.experimentId = experimentId;
            this.user = user;
        }
        
        public String getBadge() {
            return user.getBadge();
        }
        
        public UserInfo getUser() {
            return user;
        }        
                
        public String getGlobusUsername() {
            return user.getGlobusUsername();
        }

        public String getUsername() {
            return user.getUsername();
        }

         public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public boolean isIsManager() {
            return userRoles[experimentRoles.get(RoleTypeName.MANAGER).getId()];
        }

        public void setIsManager(boolean isManager) {
            userRoles[experimentRoles.get(RoleTypeName.MANAGER).getId()] = isManager;
        }

        public boolean isIsPI() {
            return userRoles[experimentRoles.get(RoleTypeName.PI).getId()];
        }

        public void setIsPI(boolean isPI) {
            userRoles[experimentRoles.get(RoleTypeName.PI).getId()] = isPI;
        }

        public boolean isIsUser() {
            return userRoles[experimentRoles.get(RoleTypeName.USER).getId()];
        }

        public void setIsUser(boolean isUser) {
            userRoles[experimentRoles.get(RoleTypeName.USER).getId()] = isUser;
        }

        public void setIsInRole(RoleType role, boolean isInRole) {
            userRoles[role.getId()] = isInRole;
        }

        public boolean[] getRoles() {
            return userRoles;
        }
        
        @Override
        public int hashCode() {
            return user.getId() + experimentId * 100;
        }

        @Override
        public boolean equals(Object object) {
            // TODO: Warning - this method won't work in the case the id fields are not set
            if (!(object instanceof ExperimentUser)) {
                return false;
            }
            return ((user.getId() == ((ExperimentUser) object).user.getId()) && (experimentId == ((ExperimentUser) object).experimentId));
        }
    }

    @SessionScoped
    class ExperimentUsersTable extends DataTableController<ExperimentUser> {

        @Override
        public String getEntityName() {
            return "ExperimentUser";
        }

        @Override
        public List<ExperimentUser> findAll() {
            List<UserInfo> list = userInfoFacade.findUsersInExperiment(getCurrent().getId());
            return convertExperimentUsers(list);
        }
        
        List<ExperimentUser> convertExperimentUsers(List<UserInfo> list) {
            if (!initialized) {
                initializeTables();
            }
            logger.debug("converting ExperimentUser ");
            ExperimentUser experimentUser;
            for (UserInfo user : list) {
                if ((experimentUser = experimentUsers.get(user)) == null) {
                    experimentUser = new ExperimentUser(getCurrent().getId(), user);
                    experimentUser.setName(user.getLastName() + ", " + user.getFirstName());
                    experimentUsers.put(user, experimentUser);
                }
                for (RoleType roleType : experimentRoles.values()) {
                    boolean inRole = (user.getExperimentRole(roleType.getId(), getCurrent().getId()) != null);
                    experimentUser.setIsInRole(roleType, inRole);
                }
            }
            return new ArrayList<>(experimentUsers.values());
        }
        
        @Override
        public String getTableName() {
            return "experimentUsersTable";
        }
    }

    @SessionScoped
    class NoExperimentUsersTypeTable extends DataTableController<UserInfo> {

        @Override
        public String getEntityName() {
            return "UserInfo";
        }

        @Override
        public List<UserInfo> findAll() {
            resetList();
            return userInfoFacade.findNoUsersInExperiment(getCurrent().getId());
        }
        
        @Override
        public String getTableName() {
            return "noExperimentUsersTable";
        }
    }

    @SessionScoped
    class ExperimentPolicyTable extends DataTableController<ExperimentPolicy> {

        boolean forEdit;
        
        ExperimentPolicyTable(boolean forEdit) {
            this.forEdit = forEdit;
        }
        
        @Override
        public String getEntityName() {
            return "ExperimentPolicy";
        }

        @Override
        public List<ExperimentPolicy> findAll() {
            resetList();
            List<ExperimentPolicy> experimentPolicies = experimentPolicyFacade.findByExperimentId(current.getId());
            if (forEdit) {
                experimentPropertiesMap.clear();
                for (ExperimentPolicy experimentPolicy : experimentPolicies) {
                    for (ExperimentPolicyPropertyValue experimentPolicyPropertyValue : experimentPolicy.getExperimentPolicyPropertyValueList()) {
                        experimentPropertiesMap.put(experimentPolicyPropertyValue.getId(), experimentPolicyPropertyValue.getPolicyPropertyValue());
                    }
                }
            }
            return experimentPolicies;
        }
        
        @Override
        public String getTableName() {
            return "experimentPolicyTable";
        }        
    }

    public class PolicyPropertyValue extends CloneableEntity {

        private PolicyProperty policyProperty;
        private String value;

        public PolicyProperty getPolicyProperty() {
            return policyProperty;
        }

        public void setPolicyProperty(PolicyProperty policyProperty) {
            this.policyProperty = policyProperty;
        }

        public String getValue() {
            return value;
        }

        public void setValue(String value) {
            this.value = value;
        }

        public boolean hasAllowedValues() {
            return (policyProperty.getAllowedPolicyValueList() != null) && (policyProperty.getAllowedPolicyValueList().size() > 0);
        }

        public int getId() {
            return policyProperty.getId();
        }
    }

    public class MissingPolicyType extends CloneableEntity {

        private List<PolicyPropertyValue> noPropertyList = new ArrayList<>();
        private final int id;
        private final String name;

        MissingPolicyType(int id, String name) {
            this.id = id;
            this.name = name;
        }

        public List<PolicyPropertyValue> getNoPropertyList() {
            return noPropertyList;
        }

        public void setNoPropertyList(List<PolicyPropertyValue> noPropertyList) {
            this.noPropertyList = noPropertyList;
        }

        public int getId() {
            return id;
        }

        public String getName() {
            return name;
        }

        protected void clear() {
            noPropertyList.clear();
        }

        protected boolean isEmpty() {
            return noPropertyList.isEmpty();
        }
    }

    @SessionScoped
    public class NoPolicyValueTable extends DataTableController<PolicyPropertyValue> {

       NoPolicyValueTable() {          
       }
 
        @Override
        public String getEntityName() {
            return "PolicyPropertyValue";
        }

        @Override
        public List<PolicyPropertyValue> findAll() {
            setMissingProperties(experimentPolicyFacade.findByExperimentId(current.getId()), policyTypeFacade.findAll());
            List<PolicyPropertyValue> missingProperties = new ArrayList<>();
            for (MissingPolicyType missingPolicyType : missingPolicyTypes) {
                missingProperties.addAll(missingPolicyType.getNoPropertyList());
            }
            return missingProperties;
        }       

        @Override
        public String getTableName() {
            return "noPolicyValueTable";
        }
     }
  
    private final int rows = 25;
    private ExperimentPolicyPropertyValue experimentPropertyToDelete;
    private ExperimentController.ExperimentUsersTable experimentUsersListTable = new ExperimentController.ExperimentUsersTable();
    private ExperimentController.ExperimentUsersTable experimentUsersEditTable = new ExperimentController.ExperimentUsersTable();
    private ExperimentController.NoExperimentUsersTypeTable noExperimentUsersTypeTable = new ExperimentController.NoExperimentUsersTypeTable();
    private ExperimentController.ExperimentPolicyTable experimentPolicyListTable = new ExperimentController.ExperimentPolicyTable(false);
    private ExperimentController.ExperimentPolicyTable experimentPolicyEditTable = new ExperimentController.ExperimentPolicyTable(true);
    private ExperimentController.NoPolicyValueTable noPolicyValueTable = new ExperimentController.NoPolicyValueTable();
    final private Map<UserInfo, ExperimentUser> experimentUsers = new HashMap<>();
    final private Map<String, RoleType> experimentRoles = new HashMap<>();
    int maxExperimentRoleTypeId = 0;
    boolean initialized = false;
    private List<MissingPolicyType> missingPolicyTypes = new ArrayList<>();
    private final Map<Integer, String> experimentPropertiesMap = new HashMap<>();
    private final List<String> notEditableTypes = new ArrayList<>();
    { notEditableTypes.add("ESAF"); }

    public ExperimentController() {
    }

    @Override
    public void clear() {
        experimentUsersListTable.resetList();
        experimentUsersEditTable.resetList();
        experimentUsers.clear();
        noExperimentUsersTypeTable.resetList();
        experimentPolicyListTable.resetList();
        missingPolicyTypes.clear();
        experimentPolicyEditTable.resetList();
        noPolicyValueTable.resetList();
        experimentPropertyToDelete = null;
    }
    
    @Override
    protected ExperimentFacade getFacade() {
        return experimentFacade;
    }

    @Override
    protected Experiment createEntityInstance() {
        Experiment newExperiment = new Experiment();
        if (!initialized) {
            initializeTables();
        }
        return newExperiment;
    }

    @Override
    public String getEntityTypeName() {
        return "experiment";
    }

    @Override
    public String getCurrentEntityInstanceName() {
        if (getCurrent() != null) {
            return getCurrent().getName();
        }
        return "";
    }

    private boolean isAdmin(String username) {
        return roleTypeFacade.findByName(RoleTypeName.ADMIN).isAdmin(username);
    }
    
    @Override
    public DataModel createListDataModel() {
        UserInfo logged = (UserInfo) SessionUtility.getUser();
        if (logged == null) {
            return null;
        }
        if (isAdmin(logged.getUsername())) {
            return new ListDataModel(getFacade().findAll());
        } else {
            return new ListDataModel(userInfoFacade.findExperimentsInUser(logged.getId()));
        }
    }

    @Override
    public List<Experiment> getAvailableItems() {
        UserInfo logged = (UserInfo) SessionUtility.getUser();
        if (logged == null) {
            return null;
        }
        if (isAdmin(logged.getUsername())) {
            return getFacade().findAll();
        } else {
            return userInfoFacade.findExperimentsInUser(logged.getId());
        }
    }

    @Override
    public DataModel createSelectDataModel() {
        UserInfo logged = (UserInfo) SessionUtility.getUser();
        if (logged == null) {
            return null;
        }
        List<Experiment> selectEntityList;
        if (isAdmin(logged.getUsername())) {
            selectEntityList = getFacade().findAll();
        } else {
            selectEntityList = userInfoFacade.findExperimentsInUser(logged.getId());
        }
        prepareEntityListForSelection(selectEntityList);
        return new ListDataModel(selectEntityList);
    }

    @Override
    public SelectItem[] getAvailableItemsForSelectMany() {
        UserInfo logged = (UserInfo) SessionUtility.getUser();
        if (logged == null) {
            return null;
        }
        if (isAdmin(logged.getUsername())) {
            return CollectionUtility.getSelectItems(getFacade().findAll(), false);
        } else {
            return CollectionUtility.getSelectItems(userInfoFacade.findExperimentsInUser(logged.getId()), false);
        }
    }

    @Override
    public SelectItem[] getAvailableItemsForSelectOne() {
        UserInfo logged = (UserInfo) SessionUtility.getUser();
        if (logged == null) {
            return null;
        }
        if (isAdmin(logged.getUsername())) {
            return CollectionUtility.getSelectItems(getFacade().findAll(), true);
        } else {
            return CollectionUtility.getSelectItems(userInfoFacade.findExperimentsInUser(logged.getId()), true);
        }
    }

    @Override
    public String prepareEdit(Experiment experiment) {
        clear();
        if (!initialized) {
            initializeTables();
        }
        return super.prepareEdit(experiment);
    }

    private void initializeTables() {
        List<RoleType> roleTypesList = roleTypeFacade.findAll();
        for (RoleType roleType : roleTypesList) {
            if (!roleType.isIsSystemRole()) {
                experimentRoles.put(roleType.getName(), roleType);
                if (roleType.getId() > maxExperimentRoleTypeId) {
                    maxExperimentRoleTypeId = roleType.getId();
                }
            }
        }
        initialized = true;
    }
    
    @Override
    public void prepareEntityInsert(Experiment experiment) throws ObjectAlreadyExists, InvalidDate, MissingProperty {
        if ((experiment.getName() == null) || (experiment.getName().length() == 0)){
            throw new MissingProperty("Experiment name is missing.");
        }
        Experiment existingExperiment = experimentFacade.findByName(experiment.getName());
        if (existingExperiment != null) {
            throw new ObjectAlreadyExists("Experiment " + experiment.getName() + " already exists.");
        }

        verifyExperiment(experiment);
        logger.debug("Inserting new experiment " + experiment.getName());
    }

    @Override
    public void prepareEntityUpdate(Experiment experiment) throws DmPortalException {
        if ((experiment.getName() == null) || (experiment.getName().length() == 0)) {
            throw new MissingProperty("Experiment name is missing.");
        }
        verifyExperiment(experiment);        
        logger.debug("Updating experiment " + experiment.getName());                
    }

     @Override
     protected String getObjectAlreadyExistMessage(Experiment experiment)  {
        if (experiment == null) {
            return null;
        }
        return "Experiment " + experiment.getName() + " already exists."; 
    }
     
    private void verifyExperiment(Experiment experiment) throws InvalidDate, MissingProperty {       
        if (experiment.getExperimentType() == null) {
            throw new MissingProperty("Experiment Type is missing.");
        }
        
        if ((experiment.getStartDate() != null) && (experiment.getEndDate() != null) && (experiment.getEndDate().before(experiment.getStartDate()))) {
            throw new InvalidDate("Experiment end date is before start date.");
        }
    }
   
    @Override
    public String update() {
        boolean updated;
        for (UserInfo user : experimentUsers.keySet()) {
            updated = false;
            ExperimentUser experimentUser = experimentUsers.get(user);
            for (RoleType roleType : experimentRoles.values()) {
                if (!roleType.isIsSystemRole()) {
                    int roleId = roleType.getId();
                    if (user.getExperimentRole(roleId, experimentUser.experimentId) == null) {
                        if (experimentUser.userRoles[roleId]) {
                            UserExperimentRole userExperimentRole = new UserExperimentRole(user.getId(), experimentUser.experimentId, roleId);
                            user.addUserExperimentRole(userExperimentRole);
                            updated = true;
                            logger.debug("adding userExperimentRole " + user.getId() + " " + experimentUser.experimentId + " " + roleId);
                        }
                    } else if (!experimentUser.userRoles[roleId]) {
                        user.removeExperimentRole(roleId, experimentUser.experimentId);
                        updated = true;
                        logger.debug("adding userExperimentRole " + user.getId() + " " + experimentUser.experimentId + " " + roleId);
                    }
                }
                if (updated) {
                    userInfoFacade.edit(user);
                }
            }
        }

        Iterator<ExperimentPolicy> iterator = experimentPolicyEditTable.getListDataModel().iterator();
        while (iterator.hasNext()) {
            ExperimentPolicy experimentPolicy = iterator.next();        
            for (ExperimentPolicyPropertyValue experimentPolicyPropertyValue : experimentPolicy.getExperimentPolicyPropertyValueList()) {
                String oldValue = experimentPropertiesMap.get(experimentPolicyPropertyValue.getId());
                String newValue = experimentPolicyPropertyValue.getPolicyPropertyValue();
                if ((oldValue != null) && !oldValue.equals(experimentPolicyPropertyValue.getPolicyPropertyValue())) {
                    experimentPolicyPropertyValue.setModifiedBy(((UserInfo) SessionUtility.getUser()).getUsername());
                    experimentPolicyPropertyValue.setModifiedDate(new Date());
                    experimentPolicyPropertyValue.setPolicyPropertyValue(newValue);
                    experimentPolicyPropertyValueFacade.edit(experimentPolicyPropertyValue);
                }
            }
        }
        
        // Notify DS Web Service
        try {
            ExperimentDsApi api = DmApiFactory.getExperimentDsApi();
            api.updateExperiment(getCurrent().getName());
        }
        catch (DmException ex) {
            logger.error("Could not notify Data Storage Service: " + ex);
            SessionUtility.addErrorMessage("Error", "Unable to notify Data Storage Service: " + ex.getErrorMessage());
        }
        clear();
        return super.update();
    }
    
    void setMissingProperties(List<ExperimentPolicy> experimentPolicies, List<PolicyType> policyTypes) {
        for (PolicyType policyType : policyTypes) {
            ExperimentPolicy theExperimentPolicy = null;
            boolean missingProperties = true;
            for (ExperimentPolicy experimentPolicy : experimentPolicies) {
                if (experimentPolicy.getPolicyType().getId() == policyType.getId()) {
                    theExperimentPolicy = experimentPolicy;
                    if (theExperimentPolicy.getExperimentPolicyPropertyValueList().size() == policyType.getPolicyPropertyList().size()) {
                        missingProperties = false;
                    }
                    break;
                }
            }
            if (missingProperties) {
                MissingPolicyType missingPolicyType = new MissingPolicyType(policyType.getId(), policyType.getName());
                missingPolicyTypes.add(missingPolicyType);
                for (PolicyProperty policyProperty : policyType.getPolicyPropertyList()) {
                    if ((theExperimentPolicy == null) || (!hasProperty(theExperimentPolicy, policyProperty))) {
                        PolicyPropertyValue policyPropertyValue = new PolicyPropertyValue();
                        policyPropertyValue.policyProperty = policyProperty;
                        policyPropertyValue.value = policyProperty.getDefaultValue();
                        missingPolicyType.getNoPropertyList().add(policyPropertyValue);
                    }
                }
            }
        }
    }

    private boolean hasProperty(ExperimentPolicy experimentPolicy, PolicyProperty policyProperty) {
        for (ExperimentPolicyPropertyValue experimentPolicyPropertyValue : experimentPolicy.getExperimentPolicyPropertyValueList()) {
            if (experimentPolicyPropertyValue.getPolicyProperty().equals(policyProperty)) {
                return true;
            }
        }
        return false;
    }

       public void updateRemovedExperimentRoles() {
        UserExperimentRole userExperimentRole;
        for (UserInfo user : experimentUsers.keySet()) {
            ExperimentUser experimentUser = experimentUsers.get(user);
            for (RoleType roleType : experimentRoles.values()) {
                int roleId = roleType.getId();
                userExperimentRole = user.getExperimentRole(roleId, experimentUser.experimentId);
                if ((userExperimentRole != null ) && (!experimentUser.userRoles[roleId]) ) {
                   userExperimentRoleFacade.remove(userExperimentRole);
                }
            }
        }
    }

   public String getRemovedUserName() {
        if (experimentUsersEditTable.getCurrentObject() != null) {
            return experimentUsersEditTable.getCurrentObject().getUsername();
        }
        return "";
    }
        
   public String getRemovedPolicyName() {
       if (experimentPropertyToDelete != null) {
            return experimentPropertyToDelete.getPolicyProperty().getName();
       }
       return "";
    }
        
    public void removeUserRoles() {
        ExperimentUser experimentUser = experimentUsersEditTable.getCurrentObject();
        UserInfo user = experimentUser.getUser();
        List<UserExperimentRole> roleList = user.getUserExperimentRoles(experimentUser.experimentId);
        for (UserExperimentRole userExperimentRole : roleList) {
             userExperimentRoleFacade.remove(userExperimentRole); 
        }
    }
    
    public String removeUser() {
        ExperimentUser experimentUser = experimentUsersEditTable.getCurrentObject();
        UserInfo user = experimentUser.getUser();
        for (RoleType roleType : experimentRoles.values()) {
            int roleId = roleType.getId();
            if (experimentUser.userRoles[roleId]) {
                user.removeExperimentRole(roleId, experimentUser.experimentId);
            }
        }
        userInfoFacade.edit(user);       

        experimentUsersListTable.resetList();
        experimentUsersEditTable.resetList();
        experimentUsers.clear();
        noExperimentUsersTypeTable.resetList();

        return "edit?faces-redirect=true";
    }
    
    public void removePolicy() {
        experimentPolicyPropertyValueFacade.remove(experimentPropertyToDelete);
    }
    
    public String updateRemovedExperimentPolicy() {
        ExperimentPolicy experimentPolicy = experimentPropertyToDelete.getExperimentPolicy();
        experimentPropertiesMap.remove(experimentPropertyToDelete.getId());
        experimentPolicy.getExperimentPolicyPropertyValueList().remove(experimentPropertyToDelete);
        experimentPolicyFacade.edit(experimentPolicy);
        current = experimentFacade.find(current.getId());
        experimentPolicyListTable.resetList();
        missingPolicyTypes.clear();
        experimentPolicyEditTable.resetList();
        noPolicyValueTable.resetList();
        return "edit?faces-redirect=true";
    }

    public String addExperimentUser() {
        List<UserInfo> newUsers = noExperimentUsersTypeTable.getSelectedObjectList();
        if (newUsers == null) {
            logger.debug("null selected list");
            return null;
        } else if (newUsers.isEmpty()) {
            logger.debug("empty selected list");
            return null;
        } else {
            try {
                UserExperimentRole userExperimentRole;
                for (UserInfo user : newUsers) {
                    userExperimentRole = new UserExperimentRole(user.getId(), current.getId(), experimentRoles.get(RoleTypeName.USER).getId()); // set the User
                    user.getUserExperimentRoleList().add(userExperimentRole);
                    userInfoFacade.edit(user);
                }
                experimentUsersListTable.resetList();
                experimentUsersEditTable.resetList();
                experimentUsers.clear();
                noExperimentUsersTypeTable.resetList();
                return "edit?faces-redirect=true";
            } catch (RuntimeException ex) {
                SessionUtility.addErrorMessage("Error", "Could not update UserInfo" + ": " + ex.getMessage());
                return null;
            }
        }
    }

    public String addExperimentPolicyPropertyValues() {
        List<PolicyPropertyValue> newProperties = noPolicyValueTable.getSelectedObjectList();
        if (newProperties == null) {
            logger.debug("null selected list");
            return null;
        } else if (newProperties.isEmpty()) {
            logger.debug("empty selected list");
            return null;
        } else {
            try {
                ExperimentPolicyPropertyValue experimentPolicyPropertyValue;
                for (PolicyPropertyValue property : newProperties) {
                    PolicyProperty policyProperty = property.getPolicyProperty();
                    experimentPolicyPropertyValue = new ExperimentPolicyPropertyValue();
                    experimentPolicyPropertyValue.setPolicyProperty(policyProperty);
                    experimentPolicyPropertyValue.setPolicyPropertyValue(property.getValue());
                    experimentPolicyPropertyValue.setModifiedBy(((UserInfo) SessionUtility.getUser()).getUsername());
                    experimentPolicyPropertyValue.setModifiedDate(new Date());
                    ExperimentPolicy experimentPolicy = getOrCreateExperimentPolicy(policyProperty.getPolicyType());
                    experimentPolicyPropertyValue.setExperimentPolicy(experimentPolicy);
                    experimentPolicy.getExperimentPolicyPropertyValueList().add(experimentPolicyPropertyValue);
                }
                super.update();
                current = experimentFacade.find(current.getId());
                experimentPolicyListTable.resetList();
                missingPolicyTypes.clear();
                experimentPolicyEditTable.resetList();
                noPolicyValueTable.resetList();

                return "edit?faces-redirect=true";
            } catch (RuntimeException ex) {
                SessionUtility.addErrorMessage("Error", "Could not update Experiment Policy" + ": " + ex.getMessage());
                return null;
            }
        }        
    }
  
    private ExperimentPolicy getOrCreateExperimentPolicy(PolicyType policyType) {
        List<ExperimentPolicy> experimentPolicyList = current.getExperimentPolicyList();
        for (ExperimentPolicy experimentPolicy: experimentPolicyList) {
            if (experimentPolicy.getPolicyType().equals(policyType)) {
                return experimentPolicy;
            }
        }
        ExperimentPolicy newExperimentPolicy = new ExperimentPolicy();
        newExperimentPolicy.setExperiment(current);
        newExperimentPolicy.setPolicyType(policyType);
        experimentPolicyList.add(newExperimentPolicy);
        policyType.getExperimentPolicyList().add(newExperimentPolicy);
        experimentPolicyFacade.create(newExperimentPolicy);
        return newExperimentPolicy;
    }
    
    public boolean canEditExperiment(Experiment experiment) {
        UserInfo logged = (UserInfo) SessionUtility.getUser();
        if (logged == null) {
            return false;
        }
        if (experiment == null) {
            return false;
        }
        if (!initialized) {
            initializeTables();
        }
        // user that is Manager or PI can edit experiment
        int managerRoleId = experimentRoles.get(RoleTypeName.MANAGER).getId();
        int piRoleId = experimentRoles.get(RoleTypeName.PI).getId();
        return (logged.getExperimentRole(managerRoleId, experiment.getId()) != null) || (logged.getExperimentRole(piRoleId, experiment.getId()) != null);
    }
    
    public boolean isRestricted() {
        UserInfo logged = (UserInfo) SessionUtility.getUser();
        if (logged == null) {
            return false;
        }
        if (current == null) {
            return true;
        }
        int managerRoleId = experimentRoles.get(RoleTypeName.MANAGER).getId();
        int piRoleId = experimentRoles.get(RoleTypeName.PI).getId();
        return (logged.getExperimentRole(managerRoleId, current.getId()) == null) && (logged.getExperimentRole(piRoleId, current.getId()) == null);
    }
    
    public boolean canDeleteExperiment(Experiment experiment) {
        UserInfo logged = (UserInfo) SessionUtility.getUser();
        if (logged == null) {
            return false;
        }
        if (experiment == null) {
            return false;
        }
        if (!initialized) {
            initializeTables();
        }
        int managerRoleId = experimentRoles.get(RoleTypeName.MANAGER).getId();
        return logged.getExperimentRole(managerRoleId, experiment.getId()) != null;
    }
    
    public boolean canAddManager() {
       ExperimentUser logged = experimentUsers.get((UserInfo) SessionUtility.getUser());
        if (logged == null) {
            return false;
        }
        return logged.isIsManager();
    }
    
    public boolean canAddPiAndUser() {
       ExperimentUser logged = experimentUsers.get((UserInfo) SessionUtility.getUser());
         if (logged == null) {
            return false;
        }
        return logged.isIsManager() || logged.isIsPI();
    }

    public boolean notSelected() {
        return (current == null);
    }

    public int getRows() {
        return rows;
    }
    
    public ExperimentController.ExperimentUsersTable getExperimentUsersListTable() {
        return experimentUsersListTable;
    }

    public void setExperimentUsersListTable(ExperimentController.ExperimentUsersTable experimentUsersTable) {
        this.experimentUsersListTable = experimentUsersTable;
    }
    
    public ExperimentController.ExperimentUsersTable getExperimentUsersEditTable() {
        return experimentUsersEditTable;
    }

    public void setExperimentUsersEditTable(ExperimentController.ExperimentUsersTable experimentUsersTable) {
        this.experimentUsersEditTable = experimentUsersTable;
    }
    
    public ExperimentController.NoExperimentUsersTypeTable getNoExperimentUsersTypeTable() {
        return noExperimentUsersTypeTable;
    }

    public void setNoExperimentUsersTypeTable(ExperimentController.NoExperimentUsersTypeTable noExperimentUsersTypeTable) {
        this.noExperimentUsersTypeTable = noExperimentUsersTypeTable;
    }

    public ExperimentPolicyTable getExperimentPolicyListTable() {
        return experimentPolicyListTable;
    }

    public void setExperimentPolicyListTable(ExperimentPolicyTable experimentPolicyTable) {
        this.experimentPolicyListTable = experimentPolicyTable;
    }

    public ExperimentPolicyTable getExperimentPolicyEditTable() {
        return experimentPolicyEditTable;
    }

    public void setExperimentPolicyEditTable(ExperimentPolicyTable experimentPolicyEditTable) {
        this.experimentPolicyEditTable = experimentPolicyEditTable;
    }

    public NoPolicyValueTable getNoPolicyValueTable() {
        return noPolicyValueTable;
    }

    public void setNoPolicyValueTable(NoPolicyValueTable noPolicyValueTable) {
        this.noPolicyValueTable = noPolicyValueTable;
    }

    public List<MissingPolicyType> getMissingPolicyTypes() {
        return missingPolicyTypes;
    }

    public void setMissingPolicyTypes(List<MissingPolicyType> missingPolicyTypes) {
        this.missingPolicyTypes = missingPolicyTypes;
    }

    public ExperimentPolicyPropertyValue getExperimentPropertyToDelete() {
        return experimentPropertyToDelete;
    }

    public void setExperimentPropertyToDelete(ExperimentPolicyPropertyValue experimentPropertyToDelete) {
        this.experimentPropertyToDelete = experimentPropertyToDelete;
    }
    
    public boolean isEditable() {
        return notEditableTypes.contains(current.getExperimentType().getName());
    }

     @FacesConverter(forClass = Experiment.class)
    public static class ExperimentControllerConverter implements Converter {

        @Override
        public Object getAsObject(FacesContext facesContext, UIComponent component, String value) {
            if (value == null || value.length() == 0) {
                return null;
            }
            ExperimentController controller = (ExperimentController) facesContext.getApplication().getELResolver().
                    getValue(facesContext.getELContext(), null, "experimentController");
            return controller.getEntity(getKey(value));
        }

        java.lang.Integer getKey(String value) {
            java.lang.Integer key;
            key = Integer.valueOf(value);
            return key;
        }

        String getStringKey(java.lang.Integer value) {
            StringBuilder sb = new StringBuilder();
            sb.append(value);
            return sb.toString();
        }

        @Override
        public String getAsString(FacesContext facesContext, UIComponent component, Object object) {
            if (object == null) {
                return null;
            }
            if (object instanceof Experiment) {
                Experiment o = (Experiment) object;
                return getStringKey(o.getId());
            }
            else {
                throw new IllegalArgumentException("object " + object + " is of type " + object.getClass().getName() + "; expected type: " + Experiment.class.getName());
            }
        }

    }
   
}