/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package gov.anl.aps.dm.portal.controllers;

import gov.anl.aps.dm.portal.exceptions.DmPortalException;
import gov.anl.aps.dm.portal.exceptions.MissingProperty;
import gov.anl.aps.dm.portal.exceptions.ObjectAlreadyExists;
import gov.anl.aps.dm.portal.model.beans.PolicyPropertyFacade;
import gov.anl.aps.dm.portal.model.entities.PolicyProperty;
import gov.anl.aps.dm.portal.model.entities.PolicyType;
import java.util.List;
import javax.ejb.EJB;
import javax.enterprise.context.SessionScoped;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.FacesConverter;
import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;
import javax.inject.Named;
import org.apache.log4j.Logger;

/**
 *
 * @author bfrosik
 */
@Named("policyPropertyController")
@SessionScoped
public class PolicyPropertyController extends CrudEntityController<PolicyProperty, PolicyPropertyFacade>
{
    
    private PolicyType policyType;
    private int rows = 25;

    private static final Logger logger = Logger.getLogger(PolicyPropertyController.class.getName());

    @EJB
    private PolicyPropertyFacade policyPropertyFacade;

    class PolicyPropertyInfoTable extends DataTableController<PolicyProperty> {

        @Override
        public String getEntityName() {
            return "PolicyProperty";
        }

        @Override
        public List<PolicyProperty> findAll() {
            return policyPropertyFacade.findByPolicyTypeId(policyType.getId());
        }

        @Override
        public String getTableName() {
            return "policyPropertyInfoTable";
        }
    }
    
    private PolicyPropertyInfoTable policyPropertyInfoTable = new PolicyPropertyInfoTable();

    public PolicyPropertyController() {
    }

    public PolicyPropertyInfoTable getPolicyPropertyInfoTable() {
        return policyPropertyInfoTable;
    }

    public void setPolicyPropertyInfoTable(PolicyPropertyInfoTable policyPropertyInfoTable) {
        this.policyPropertyInfoTable = policyPropertyInfoTable;
    }

    @Override
    protected PolicyPropertyFacade getFacade() {
        return policyPropertyFacade;
    }

    @Override
    protected PolicyProperty createEntityInstance() {
        return new PolicyProperty();
    }

    @Override
    public String getCurrentEntityInstanceName() {
        if (getCurrent() != null) {
            return getCurrent().getName();
        }
        return "";
    }

    @Override
    public DataModel createListDataModel() {
        return new ListDataModel((List) policyPropertyFacade.findByPolicyTypeId(policyType.getId()));
    }

    @Override
    public List<PolicyProperty> getAvailableItems() {
        return (List<PolicyProperty>) policyPropertyFacade.findByPolicyTypeId(policyType.getId());
    }

    @Override
    public String prepareEdit(PolicyProperty policyProperty) {
        super.prepareEdit(policyProperty);
        return "/views/policyProperty/edit?faces-redirect=true";
    }

    @Override
    public String prepareView(PolicyProperty policyProperty) {
        super.prepareView(policyProperty);
        return "/views/policyProperty/view?faces-redirect=true";
    }

    public String prepareView() {
        return super.prepareView(current);
    }

    @Override
    public void prepareEntityInsert(PolicyProperty policyProperty) throws ObjectAlreadyExists, MissingProperty {
        current.setPolicyType(policyType);
        if ((policyProperty.getName() == null) || (policyProperty.getName().length() == 0)) {
            throw new MissingProperty("Name is missing.");
        }
        List<PolicyProperty> policyPropertyList = policyPropertyFacade.findByPolicyTypeId(policyType.getId());
        for (PolicyProperty property : policyPropertyList) {
            if ((property.getName().equals(policyProperty.getName()))) {
                throw new ObjectAlreadyExists("policy property " + policyProperty.getName() + " already exists for this policy type.");
            }
        }
        logger.debug("Inserting new policy property " + policyProperty.getName());
    }

    @Override
    public void prepareEntityUpdate(PolicyProperty policyProperty) throws DmPortalException {       
        current.setPolicyType(policyType);
    }
    
     @Override
     protected String getObjectAlreadyExistMessage(PolicyProperty policyProperty)  {
        if (policyProperty == null) {
            return null;
        }
        return "Policy Property " + policyProperty.getName() + " already exists."; 
    }

    @Override
    public String getEntityTypeName() {
        return "PolicyProperty";
    }
    
    @Override
    public void clear() {
        policyPropertyInfoTable.resetList();
        resetList();
    }

    @Override
    public String prepareCreate() {
        super.prepareCreate();
        return "/views/policyProperty/create?faces-redirect=true";
    }
    
    @Override
    public String create() {
        if (super.create() == null) {
            return null;
        }
        policyPropertyInfoTable.resetList();
        return "/views/policyType/view?faces-redirect=true";
    }
    
    @Override
    public String prepareList() {
        logger.debug("Preparing list");
        resetListDataModel();
        return "/views/policyType/view?faces-redirect=true";
    }
    
//    @Override
//    public String destroy() {
//        String view = super.destroy();
//        clear();
//        return view;
//    }
    
    public PolicyType getPolicyType() {
        return policyType;
    }

    public void setPolicyType(PolicyType policyType) {
        logger.info("setting policy type ");
        clear();
        this.policyType = policyType;
    }

    public int getRows() {
        return rows;
    }

    public void setRows(int rows) {
        this.rows = rows;
    }
    
    public boolean hasAllowedValues(PolicyProperty policyProperty) {
        return ((policyProperty.getAllowedPolicyValueList() != null) && !policyProperty.getAllowedPolicyValueList().isEmpty());
    }
    
    public String updateAllowedValueList() {
        policyPropertyFacade.edit(current);
        return "/views/policyProperty/edit?faces-redirect=true";
    }
    
    public boolean notSelected() {
        return current == null;
    }
 
    
    @FacesConverter(forClass = PolicyProperty.class)
    public static class PolicyPropertyControllerConverter implements Converter
    {

        @Override
        public Object getAsObject(FacesContext facesContext, UIComponent component, String value) {
            if (value == null || value.length() == 0) {
                return null;
            }
            PolicyPropertyController controller = (PolicyPropertyController) facesContext.getApplication().getELResolver().
                    getValue(facesContext.getELContext(), null, "policyPropertyController");
            return controller.getEntity(getKey(value));
        }

        java.lang.Integer getKey(String value) {
            java.lang.Integer key;
            key = Integer.valueOf(value);
            return key;
        }

        String getStringKey(java.lang.Integer value) {
            StringBuilder sb = new StringBuilder();
            sb.append(value);
            return sb.toString();
        }

        @Override
        public String getAsString(FacesContext facesContext, UIComponent component, Object object) {
            if (object == null) {
                return null;
            }
            if (object instanceof PolicyProperty) {
                PolicyProperty o = (PolicyProperty) object;
                return getStringKey(o.getId());
            }
            else {
                throw new IllegalArgumentException("object " + object + " is of type " + object.getClass().getName() + "; expected type: " + PolicyProperty.class.getName());
            }
        }
    }
}

