package gov.anl.aps.dm.portal.controllers;

import gov.anl.aps.dm.portal.exceptions.DmPortalException;
import gov.anl.aps.dm.portal.exceptions.MissingProperty;
import gov.anl.aps.dm.portal.exceptions.ObjectAlreadyExists;
import gov.anl.aps.dm.portal.model.beans.RoleTypeFacade;
import gov.anl.aps.dm.portal.model.entities.RoleType;
import java.util.List;
import javax.ejb.EJB;
import javax.enterprise.context.SessionScoped;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.FacesConverter;
import javax.inject.Named;
import org.apache.log4j.Logger;

@Named("roleTypeController")
@SessionScoped
public class RoleTypeController extends CrudEntityController<RoleType, RoleTypeFacade>
{
    private static final Logger logger = Logger.getLogger(RoleTypeController.class.getName());
    private final int rows = 25;

    @EJB
    private RoleTypeFacade roleTypeFacade;

    public RoleTypeController() {
    }
    
    public int getRows() {
        return rows;
    }
 
    @Override
    protected RoleTypeFacade getFacade() {
        return roleTypeFacade;
    }

    @Override
    protected RoleType createEntityInstance() {
        return new RoleType();
    }

    @Override
    public String getEntityTypeName() {
        return "roleType";
    }

    @Override
    public String getCurrentEntityInstanceName() {
        if (getCurrent() != null) {
            return getCurrent().getName();
        }
        return "";
    }

    @Override
    public List<RoleType> getAvailableItems() {
        return super.getAvailableItems();
    }

    @Override
    public String prepareEdit(RoleType roleType) {
        return super.prepareEdit(roleType);
    }

    @Override
    public void prepareEntityInsert(RoleType roleType) throws ObjectAlreadyExists, MissingProperty {
        if ((roleType.getName() == null) || (roleType.getName().length() == 0)) {
            throw new MissingProperty("Name is missing.");
        }
        RoleType existingRoleType = roleTypeFacade.findByName(roleType.getName());
        if (existingRoleType != null) {
            throw new ObjectAlreadyExists("Role Type " + roleType.getName() + " already exists.");
        }
        logger.debug("Inserting new roleType " + roleType.getName());
    }

    @Override
    public void prepareEntityUpdate(RoleType roleType) throws DmPortalException {
         if ((roleType.getName() == null) || (roleType.getName().length() == 0)) {
            throw new MissingProperty("Name is missing.");
        }
   }

     @Override
     protected String getObjectAlreadyExistMessage(RoleType roleType)  {
        if (roleType == null) {
            return null;
        }
        return "Role Type " + roleType.getName() + " already exists."; 
    }


    public boolean isRoleSystemType(RoleType roleType) {
        if (roleType == null) {
            logger.warn("Current row is not set");
            // Do nothing if current item is not set.
            return false;
        }

        return roleType.isIsSystemRole();
    }

    public boolean notSelected() {
        return current == null;
    }
    
    @FacesConverter(forClass = RoleType.class)
    public static class RoleTypeControllerConverter implements Converter {

        @Override
        public Object getAsObject(FacesContext facesContext, UIComponent component, String value) {
            if (value == null || value.length() == 0) {
                return null;
            }
            RoleTypeController controller = (RoleTypeController) facesContext.getApplication().getELResolver().
                    getValue(facesContext.getELContext(), null, "roleTypeController");
            return controller.getEntity(getKey(value));
        }

        java.lang.Integer getKey(String value) {
            java.lang.Integer key;
            key = Integer.valueOf(value);
            return key;
        }

        String getStringKey(java.lang.Integer value) {
            StringBuilder sb = new StringBuilder();
            sb.append(value);
            return sb.toString();
        }

        @Override
        public String getAsString(FacesContext facesContext, UIComponent component, Object object) {
            if (object == null) {
                return null;
            }
            if (object instanceof RoleType) {
                RoleType o = (RoleType) object;
                return getStringKey(o.getId());
            }
            else {
                throw new IllegalArgumentException("object " + object + " is of type " + object.getClass().getName() + "; expected type: " + RoleType.class.getName());
            }
        }

    }

}