package gov.anl.aps.dm.portal.controllers;

import gov.anl.aps.dm.portal.constants.RoleTypeName;
import gov.anl.aps.dm.portal.exceptions.DmPortalException;
import gov.anl.aps.dm.portal.exceptions.MissingProperty;
import gov.anl.aps.dm.portal.exceptions.ObjectAlreadyExists;
import gov.anl.aps.dm.portal.model.beans.RoleTypeFacade;
import gov.anl.aps.dm.portal.model.entities.UserInfo;
import gov.anl.aps.dm.portal.model.beans.UserInfoFacade;
import gov.anl.aps.dm.portal.model.entities.CloneableEntity;
import gov.anl.aps.dm.portal.model.entities.Experiment;
import gov.anl.aps.dm.portal.model.entities.RoleType;
import gov.anl.aps.dm.common.utilities.CryptUtility;
import gov.anl.aps.dm.portal.utilities.SessionUtility;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;

import java.util.List;
import java.util.Map;
import javax.ejb.EJB;
import javax.enterprise.context.SessionScoped;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.FacesConverter;
import javax.inject.Named;
import org.apache.log4j.Logger;

@Named("userInfoController")
@SessionScoped
public class UserInfoController extends CrudEntityController<UserInfo, UserInfoFacade> {

    private static final Logger logger = Logger.getLogger(UserInfoController.class.getName());
    private final int rows = 25;

    @EJB
    private UserInfoFacade userInfoFacade;

    @EJB
    private RoleTypeFacade roleTypeFacade;

    private String passwordEntry = null;

    public class ExperimentUser extends CloneableEntity {

        String experimentName;
        String description;
        int userId;
        int experimentId;
        private final boolean[] userRoles;

        public ExperimentUser(int experimentId, int userId) {
            userRoles = new boolean[maxExperimentRoleTypeId + 1];
            this.experimentId = experimentId;
            this.userId = userId;
        }

        void setExperimentName(String experimentName) {
            this.experimentName = experimentName;
        }

        void setDescription(String description) {
            this.description = description;
        }

        public String getExperimentName() {
            return experimentName;
        }

        public String getDescription() {
            return description;
        }

        public boolean isIsManager() {
            return userRoles[experimentRoles.get(RoleTypeName.MANAGER).getId()];
        }

        public boolean isIsPI() {
            return userRoles[experimentRoles.get(RoleTypeName.PI).getId()];
        }

        public boolean isIsUser() {
            return userRoles[experimentRoles.get(RoleTypeName.USER).getId()];
        }

        public void setIsInRole(RoleType role, boolean isInRole) {
            userRoles[role.getId()] = isInRole;
        }

        public boolean[] getRoles() {
            return userRoles;
        }

        @Override
        public int hashCode() {
            return userId + experimentId * 100;
        }

        @Override
        public boolean equals(Object object) {
            // TODO: Warning - this method won't work in the case the id fields are not set
            if (!(object instanceof ExperimentUser)) {
                return false;
            }
            return ((userId == ((ExperimentUser) object).userId) && (experimentId == ((ExperimentUser) object).experimentId));
        }
    }

    class UserExperimentsTable extends DataTableController<ExperimentUser> {

        @Override
        public String getEntityName() {
            return "UserExperiments";
        }

        @Override
        public List<ExperimentUser> findAll() {
            List<Experiment> list = userInfoFacade.findExperimentsInUser(getCurrent().getId());
            return convertUserExperiments(list);
        }

        List<ExperimentUser> convertUserExperiments(List<Experiment> list) {
            if (!initialized) {
                initializeRoleTypes();
            }
            userExperiments.clear();
            logger.debug("converting UserExperiments ");
            Collection<RoleType> roleTypes;
            roleTypes = experimentRoles.values();
            ExperimentUser experimentUser;
            for (Experiment experiment : list) {
                experimentUser = new ExperimentUser(experiment.getId(), getCurrent().getId());
                experimentUser.setExperimentName(experiment.getName());
                experimentUser.setDescription(experiment.getDescription());
                userExperiments.add(experimentUser);
                for (RoleType roleType : roleTypes) {
                    boolean inRole = (getCurrent().getExperimentRole(roleType.getId(), experiment.getId()) != null);
                    experimentUser.setIsInRole(roleType, inRole);
                }
            }
            return userExperiments;
        }

        @Override
        public String getTableName() {
            return "UserExperimentsTable";
        }
    }

    private UserInfoController.UserExperimentsTable userExperimentsListTable = new UserInfoController.UserExperimentsTable();

    private final List<ExperimentUser> userExperiments = new ArrayList<>();
    private final Map<String, RoleType> experimentRoles = new HashMap<>();
    int maxExperimentRoleTypeId;
    boolean initialized = false;

    public UserInfoController() {
    }

    public int getRows() {
        return rows;
    }

    private void initializeRoleTypes() {
        List<RoleType> roleTypesList = roleTypeFacade.findAll();
        for (RoleType roleType : roleTypesList) {
            if (!roleType.isIsSystemRole()) {
                experimentRoles.put(roleType.getName(), roleType);
                if (roleType.getId() > maxExperimentRoleTypeId) {
                    maxExperimentRoleTypeId = roleType.getId();
                }
            }
        }
        initialized = true;
    }

    @Override
    public void clear() {
        userExperimentsListTable.resetList();
        userExperiments.clear();
    }

    public UserExperimentsTable getUserExperimentsListTable() {
        return userExperimentsListTable;
    }

    public void setUserExperimentsListTable(UserExperimentsTable userExperimentsListTable) {
        this.userExperimentsListTable = userExperimentsListTable;
    }

    @Override
    protected UserInfoFacade getFacade() {
        return userInfoFacade;
    }

    @Override
    protected UserInfo createEntityInstance() {
        return new UserInfo();
    }

    @Override
    public String getEntityTypeName() {
        return "userInfo";
    }

    @Override
    public String getCurrentEntityInstanceName() {
        if (getCurrent() != null) {
            return getCurrent().getUsername();
        }
        return "";
    }

    @Override
    public List<UserInfo> getAvailableItems() {
        return super.getAvailableItems();
    }
    
    @Override
    public String prepareEdit(UserInfo user) {
        passwordEntry = null;
        return super.prepareEdit(user);
    }

    @Override
    public void prepareEntityInsert(UserInfo user) throws ObjectAlreadyExists, MissingProperty {
        verifyMandatoryParam(user);
        UserInfo existingUser = userInfoFacade.findByUsername(user.getUsername());
        if (existingUser != null) {
            throw new ObjectAlreadyExists("User " + user.getUsername() + " already exists.");
        }
        if (passwordEntry != null && !passwordEntry.isEmpty()) {
            logger.debug("Will crypt provided password for user " + user.getUsername());
            String cryptedPassword = CryptUtility.cryptPasswordWithPbkdf2(passwordEntry);
            user.setPassword(cryptedPassword);
        }
        passwordEntry = null;
        logger.debug("Inserting new user " + user.getUsername());
    }

    private void verifyMandatoryParam(UserInfo user) throws MissingProperty {
        int missing = 0;
        if ((user.getUsername() == null) || (user.getUsername().length() == 0)) {
            missing = 1;
        }
        if ((user.getLastName() == null) || (user.getLastName().length() == 0)) {
            missing += 2;
        }
        if ((user.getFirstName() == null) || (user.getFirstName().length() == 0)) {
            missing += 4;
        }
        if (missing > 0) {
            switch (missing) {
                case 7:
                    throw new MissingProperty("Username, last name, and first name are missing.");
                case 6:
                    throw new MissingProperty("last name, and first name are missing.");
                case 5:
                    throw new MissingProperty("Username and first name are missing.");
                case 4:
                    throw new MissingProperty("first name is missing.");
                case 3:
                    throw new MissingProperty("Username and last name are missing.");
                case 2:
                    throw new MissingProperty("last name is missing.");
                case 1:
                    throw new MissingProperty("Username is missing.");
            }
        }
    }

    @Override
    public void prepareEntityUpdate(UserInfo user) throws DmPortalException {
        verifyMandatoryParam(user);
        if (passwordEntry != null && !passwordEntry.isEmpty()) {
            logger.debug("Will crypt provided password for user " + user.getUsername());
            String cryptedPassword = CryptUtility.cryptPasswordWithPbkdf2(passwordEntry);
            user.setPassword(cryptedPassword);
        }
        passwordEntry = null;
    }

    @Override
    protected String getObjectAlreadyExistMessage(UserInfo user) {
        if (user == null) {
            return null;
        }
        return "User " + user.getUsername() + " already exists.";
    }

    public String prepareSessionUserEdit(String viewPath) {
        UserInfo sessionUser = (UserInfo) SessionUtility.getUser();
        if (sessionUser == null) {
            return null;
        }
        prepareEdit(sessionUser);
        return viewPath + "?faces-redirect=true";
    }

    public String getPasswordEntry() {
        return passwordEntry;
    }

    public void setPasswordEntry(String passwordEntry) {
        this.passwordEntry = passwordEntry;
    }

    public boolean notSelected() {
        return current == null;
    }
    
    @FacesConverter(forClass = UserInfo.class)
    public static class UserInfoControllerConverter implements Converter {

        @Override
        public Object getAsObject(FacesContext facesContext, UIComponent component, String value) {
            if (value == null || value.length() == 0) {
                return null;
            }
            UserInfoController controller = (UserInfoController) facesContext.getApplication().getELResolver().
                    getValue(facesContext.getELContext(), null, "userInfoController");
            return controller.getEntity(getKey(value));
        }

        java.lang.Integer getKey(String value) {
            java.lang.Integer key;
            key = Integer.valueOf(value);
            return key;
        }

        String getStringKey(java.lang.Integer value) {
            StringBuilder sb = new StringBuilder();
            sb.append(value);
            return sb.toString();
        }

        @Override
        public String getAsString(FacesContext facesContext, UIComponent component, Object object) {
            if (object == null) {
                return null;
            }
            if (object instanceof UserInfo) {
                UserInfo o = (UserInfo) object;
                return getStringKey(o.getId());
            } else {
                throw new IllegalArgumentException("object " + object + " is of type " + object.getClass().getName() + "; expected type: " + UserInfo.class.getName());
            }
        }

    }

}
