/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package gov.anl.aps.dm.portal.controllers;

import gov.anl.aps.dm.portal.model.beans.RoleTypeFacade;
import gov.anl.aps.dm.portal.model.beans.UserInfoFacade;
import gov.anl.aps.dm.portal.model.entities.RoleType;
import gov.anl.aps.dm.portal.model.entities.UserInfo;
import gov.anl.aps.dm.portal.utilities.SessionUtility;
import java.io.Serializable;
import java.util.List;
import javax.ejb.EJB;
import javax.enterprise.context.SessionScoped;
import javax.inject.Named;
import org.apache.log4j.Logger;

@Named("userSystemRoleController")
@SessionScoped
public class UserSystemRoleController  implements Serializable
{
 
    @EJB
    private UserInfoFacade userInfoFacade;

    @EJB
    private RoleTypeFacade roleTypeFacade;
    
    private int rows = 25;

    class SystemRoleTypeTable extends DataTableController<UserInfo> {
       @Override
        public String getEntityName() {
            return "UserInfo";
        }

        @Override
        public List<UserInfo> findAll() {
            return userInfoFacade.findSystemRoleUsers(systemRole.getId());
        }

        @Override
        public String getTableName() {
            return "systemRoleTable";
        }
    }
    
    class NoSystemRoleTypeTable extends DataTableController<UserInfo> {

        @Override
        public String getEntityName() {
            return "UserInfo";
        }

        @Override
        public List<UserInfo> findAll() {
            return userInfoFacade.findNoSystemRoleUsers(systemRole.getId());
        }

        @Override
        public String getTableName() {
            return "noSystemRoleTable";
        }
        
    }
    
    private SystemRoleTypeTable systemRoleTable;
    private NoSystemRoleTypeTable noSystemRoleTable;

    private RoleType systemRole;

    private static final Logger logger = Logger.getLogger(UserSystemRoleController.class.getName());
    
    public UserSystemRoleController() {
        this.systemRoleTable = new SystemRoleTypeTable();
        this.noSystemRoleTable = new NoSystemRoleTypeTable();
    }

    public int getRows() {
        return rows;
    }

    public RoleType getSystemRole() {
        return systemRole;
    }

    public void setSystemRole(RoleType systemRole) {
        this.systemRole = systemRole;
    }

    public SystemRoleTypeTable getSystemRoleTable() {
        return systemRoleTable;
    }

    public void setSystemRoleTable(SystemRoleTypeTable systemRoleTable) {
        this.systemRoleTable = systemRoleTable;
    }

     public NoSystemRoleTypeTable getNoSystemRoleTable() {
        return noSystemRoleTable;
    }

    public void setNoSystemRoleTable(NoSystemRoleTypeTable noSystemRoleTable) {
        this.noSystemRoleTable = noSystemRoleTable;
    }
         
    public String getSystemRoleName() {
        return systemRole.getName(); 
    }

     public String resetSystemRoleUserList() {
        logger.debug("Resetting System Role User list");
        systemRoleTable.resetList();
        return "/views/userSystemRole/list?faces-redirect=true";
    }
    
     public void resetNoSystemRoleUserList() {
        logger.debug("Resetting System Role Add User list");
        noSystemRoleTable.resetList();
    }
     
    public String prepareSystemRoleView() {
        logger.debug("Preparing system role view");
        noSystemRoleTable.resetList();
        return resetSystemRoleUserList();
    }

    public String addSystemRole() {
        List<UserInfo> uiList = noSystemRoleTable.getSelectedObjectList();

        if (uiList == null) {
            logger.debug("null selected list");
        } else if (uiList.isEmpty()) {
            logger.debug("empty selected list");
        } else {
            for (UserInfo userInfo : uiList) {
                systemRole.addRoleTypeToUser(userInfo);
            }
        }
        try {
            roleTypeFacade.edit(systemRole);
            resetNoSystemRoleUserList();
            return resetSystemRoleUserList();
        } catch (RuntimeException ex) {
            SessionUtility.addErrorMessage("Error", "Could not update RoleType" + ": " + ex.getMessage());
            return null;
        }
    }

    public String removeSystemRole() {
        logger.debug("removing system role from user");
        systemRole.removeRoleTypeFromUser(systemRoleTable.currentObject);
        try {
        roleTypeFacade.edit(systemRole);
        resetNoSystemRoleUserList();
        return resetSystemRoleUserList();
        }
        catch (RuntimeException ex) {
            SessionUtility.addErrorMessage("Error", "Could not update RoleType" + ": " + ex.getMessage());
            return null;
        }        
     }

}
