#!/usr/bin/env python

from dm.cat_web_service.api.fileRestApi import FileRestApi
from dm.common.exceptions.invalidRequest import InvalidRequest
from catWebServiceSessionCli import CatWebServiceSessionCli

class AddExperimentFileCli(CatWebServiceSessionCli):
    def __init__(self):
        CatWebServiceSessionCli.__init__(self, validArgCount=self.ANY_NUMBER_OF_POSITIONAL_ARGS)
        self.addOption('', '--file', dest='fileName', help='File name.')
        self.addOption('', '--experiment', dest='experimentName', help='Experiment name.')

    def checkArgs(self):
        if self.options.experimentName is None:
            raise InvalidRequest('Experiment name must be provided.')
        if self.options.fileName is None:
            raise InvalidRequest('File name must be provided.')

    def getExperimentName(self):
        return self.options.experimentName

    def getFileName(self):
        return self.options.fileName

    def runCommand(self):
        self.parseArgs(usage="""
    dm-add-experiment-file --file=FILENAME --experiment=EXPERIMENTNAME 
        [key1:value1, key2:value2, ...]

Description:
    Adds experiment file to the metadata catalog. All provided key/value pairs 
    are interpreted as file metadata.
        """)
        self.checkArgs()
        api = FileRestApi(self.getLoginUsername(), self.getLoginPassword(), self.getServiceHost(), self.getServicePort(), self.getServiceProtocol())
        fileInfo = self.splitArgsIntoDict()
        fileInfo['experimentName'] = self.getExperimentName()
        fileInfo['fileName'] = self.getFileName()
        fileMetadata = api.addExperimentFile(fileInfo)
        print fileMetadata.getDisplayString(self.getDisplayKeys(), self.getDisplayFormat())

#######################################################################
# Run command.
if __name__ == '__main__':
    cli = AddExperimentFileCli()
    cli.run()

