#!/usr/bin/env python

import copy
import types
import re

from dm.common.exceptions.dmException import DmException
from dm.common.mongodb.api.dmMongoDbApi import DmMongoDbApi
from dm.common.mongodb.impl.fileCollection import FileCollection
from dm.common.mongodb.api.dmMongoDbApi import DmMongoDbApi
from dm.common.objects.fileMetadata import FileMetadata

class FileMongoDbApi(DmMongoDbApi):

    SYSTEM_KEY_LIST = ['_id', '_fileName', '_experimentName']

    def __init__(self):
        DmMongoDbApi.__init__(self)
        self.fileCollection = FileCollection(self.dbClient)

    @DmMongoDbApi.executeDbCall
    def addExperimentFile(self, fileInfo, **kwargs):
        fileInfo2 = self.getMongoDict(fileInfo)
        dbFileMetadata = self.fileCollection.addByUniqueKeys(fileInfo2)
        return self.toDmObject(dbFileMetadata, FileMetadata)

    @DmMongoDbApi.executeDbCall
    def getFiles(self, queryDict={}, returnFieldDict=FileCollection.ALL_FIELDS_DICT, **kwargs):
        queryDict2 = self.getMongoDict(queryDict)
        self.getLogger().debug('Converting query dict to regex patterns')
        ignoreCase = kwargs.get(self.REGEX_IGNORE_CASE_KEY, True)
        queryDict2 = self.convertStringsToRegex(queryDict2, ignoreCase)
        return self.listToDmObjects(self.fileCollection.findByQueryDict(queryDict2, returnFieldDict), FileMetadata)

    @DmMongoDbApi.executeDbCall
    def getFileById(self, id, **kwargs):
        dbFileMetadata = self.fileCollection.findById(id)
        return self.toDmObject(dbFileMetadata, FileMetadata)

    @DmMongoDbApi.executeDbCall
    def getExperimentFile(self, experimentName, fileName, **kwargs):
        queryDict = { '_fileName' : fileName, '_experimentName' : experimentName }
        dbFileMetadata = self.fileCollection.findByUniqueKeys(queryDict)
        return self.toDmObject(dbFileMetadata, FileMetadata)

    @DmMongoDbApi.executeDbCall
    def getExperimentFiles(self, experimentName, queryDict={}, returnFieldDict=FileCollection.ALL_FIELDS_DICT, **kwargs):
        queryDict2 = copy.copy(queryDict)
        queryDict2['_experimentName'] = experimentName 
        return self.getFiles(queryDict2, returnFieldDict)

    @DmMongoDbApi.executeDbCall
    def updateFileById(self, fileInfo, **kwargs):
        fileInfo2 = self.getMongoDict(fileInfo)
        dbFileMetadata = self.fileCollection.updateById(fileInfo2)
        return self.toDmObject(dbFileMetadata, FileMetadata)

    @DmMongoDbApi.executeDbCall
    def updateExperimentFile(self, fileInfo, **kwargs):
        fileInfo2 = self.getMongoDict(fileInfo)
        dbFileMetadata = self.fileCollection.updateByUniqueKeys(fileInfo2)
        return self.toDmObject(dbFileMetadata, FileMetadata)

    @DmMongoDbApi.executeDbCall
    def updateOrAddExperimentFile(self, fileInfo, **kwargs):
        fileInfo2 = self.getMongoDict(fileInfo)
        dbFileMetadata = self.fileCollection.updateOrAddByUniqueKeys(fileInfo2)
        return self.toDmObject(dbFileMetadata, FileMetadata)

#######################################################################
# Testing.
if __name__ == '__main__':
    api = FileMongoDbApi()

    fileName = 'file02'
    experimentName = 'exp-01'
    fileInfo = {'fileName' : fileName, 'intKey' : 1, 'doubleKey' : 2.0, 'stringKey' : 'myString' , 'experimentName' : experimentName}
    file = api.updateOrAddExperimentFile(fileInfo)
    print '\nADDED FILE\n', file

    import time
    t = long(time.time())
    fileName = 'f-%s' % t
    experimentName = 'exp-01'
    fileInfo = {'fileName' : fileName, 'intKey' : 1, 'doubleKey' : 2.0, 'stringKey' : 'myString' , 'dictKey' : {'a' : 'A', 'b' : 'B', 'c' : 3}, 'experimentName' : experimentName}
    file = api.updateOrAddExperimentFile(fileInfo)
    print '\nADDED FILE\n', file

    files = api.getFiles()
    for file in files:
        print 'FILE: %s\n' % file.getDictRep()

    fileInfo = {'fileName' : fileName, 'experimentName' : experimentName, 'intKey' : 101}
    file = api.updateExperimentFile(fileInfo)
    print '\nUPDATED FILE\n', file

    print '\nFILES FOR EXPERIMENT exp1: \n', api.getExperimentFiles(experimentName)
    print '\nFILES FOR EXPERIMENT exp1 with fileName=file01: \n', api.getExperimentFiles(experimentName, queryDict={'fileName':'file01'})


