#!/usr/bin/env python

from dm.common.exceptions.invalidRequest import InvalidRequest
from dm.ds_web_service.api.userRestApi import UserRestApi
from dsWebServiceSessionCli import DsWebServiceSessionCli

class AddUserExperimentRoleCli(DsWebServiceSessionCli):
    def __init__(self):
        DsWebServiceSessionCli.__init__(self)
        self.addOption('', '--username', dest='username', help='User username.')
        self.addOption('', '--experiment', dest='experimentName', help='Experiment name.')
        self.addOption('', '--role', dest='roleName', help='Role name.')

    def checkArgs(self):
        if not self.options.username:
            raise InvalidRequest('Username must be provided.')
        if not self.options.experimentName:
            raise InvalidRequest('Experiment name must be provided.')
        if not self.options.roleName:
            raise InvalidRequest('Role name must be provided.')

    def getUsername(self):
        return self.options.username

    def getExperimentName(self):
        return self.options.experimentName

    def getRoleName(self):
        return self.options.roleName

    def runCommand(self):
        self.parseArgs(usage="""
    dm-add-user-experiment-role --username=USERNAME --experiment=EXPERIMENTNAME
        --role=ROLENAME

Description:
    Assigns experiment role to the given user.
        """)
        self.checkArgs()
        api = UserRestApi(self.getLoginUsername(), self.getLoginPassword(), self.getServiceHost(), self.getServicePort(), self.getServiceProtocol())
        userExperimentRole = api.addUserExperimentRole(self.getUsername(), self.getExperimentName(), self.getRoleName())
        print userExperimentRole.getDisplayString(self.getDisplayKeys(), self.getDisplayFormat())

#######################################################################
# Run command.
if __name__ == '__main__':
    cli = AddUserExperimentRoleCli()
    cli.run()

