package gov.anl.aps.dm.sync;

import java.sql.DriverManager;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

public class PsqlConnection {
    Connection connection = null;
    private final Map<String, Timestamp> dmUsers = new HashMap<>();    // <username, lastUpdate>
    private PreparedStatement insertQuery;
    private PreparedStatement updateQuery;
    private final String userTable = "user_info";

    private Logger logger;

    public PsqlConnection(Logger logger) {
    	this.logger = logger;
    }
    
    void connect(Properties config) throws SQLException {
        try {
            Class.forName("org.postgresql.Driver");
        } catch (ClassNotFoundException e) {
            logger.log(Level.SEVERE, "Can't establish PosgreSQL Driver", e);
            System.exit(0);
        }
//        logger.log(Level.INFO, "PostgreSQL JDBC Driver Registered!");

        try {
            connection = DriverManager.getConnection(
                    config.getProperty("dm.database.connection"),
                    config.getProperty("dm.database.username"),
                    config.getProperty("dm.database.password"));
        } catch (SQLException e) {
            logger.log(Level.SEVERE, "Can't connect to PostgreSQL data base");
            throw e;
        }
    }
    
    void init() throws SQLException {
        Statement statement = null;
        try {
            statement = connection.createStatement();
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "can't create statement for postgreSQL connection");
            throw ex;
        }
        ResultSet results = null;
        try {
            if (statement != null) {
                results = statement.executeQuery("SELECT * FROM " + userTable + ";");
            }
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "can't execute SELECT query from " + userTable + " table");
            throw ex;
        }
        try {
            if (results != null) {
                while (results.next()) {
                    String username = results.getString("username");
                    Timestamp lastUpdate = results.getTimestamp("last_update");
                    dmUsers.put(username, lastUpdate);
                }
                results.close();
            }
            if (statement != null) {
                statement.close();
            }
        } catch (SQLException ex) {
            logger.log(Level.WARNING, "query results processing error ", ex);
        }
       
        // prepare statements
        try {
            insertQuery = connection.prepareStatement ("INSERT INTO user_info (username, first_name, last_name, middle_name, email, badge, is_local_user, last_update) VALUES (?, ?, ?, ?, ?, ?, ?, ?)");
            updateQuery = connection.prepareStatement ("UPDATE user_info SET email = ?, last_name = ?, last_update = ? WHERE username = ?");
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "Can't get peaparedStatement from connection ", ex);
            throw ex;
        }

    }

    protected Timestamp getUserLastUpdate(String username) {
    	return dmUsers.get(username);
    }
    boolean isUser(String username) {
        return dmUsers.containsKey(username);
    }
    
    void addUser(DmUser dmuser, Timestamp currentTime) {
        try {
            insertQuery.setString(1, dmuser.username);
            insertQuery.setString(2, dmuser.firstName);
            insertQuery.setString(3, dmuser.lastName);
            insertQuery.setString(4, dmuser.middleName);
            insertQuery.setString(5, dmuser.email);
            insertQuery.setString(6, dmuser.badge);
            insertQuery.setBoolean(7, false);
            insertQuery.setTimestamp(8, currentTime);
            insertQuery.execute();
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "can't execute query to add user ", ex);
        }
    }
 
    void updateUser(DmUser dmuser, Timestamp currentTime) {
        try {
            updateQuery.setString(1, dmuser.email);
            updateQuery.setString(2, dmuser.lastName);
            updateQuery.setTimestamp(3, currentTime);
            updateQuery.setString(4, dmuser.username);
            updateQuery.executeUpdate();
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "can't execute query to update user", ex);
        }
    }
 
    void close() {
        try {
            insertQuery.close();
            updateQuery.close();
            connection.close();
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "problem closing PostgreSQL connection ", ex);
        }
    }
}
