package gov.anl.dm.esafsync;

import gov.anl.dm.esafsync.serviceconn.DaqServiceConnection;
import gov.anl.dm.esafsync.serviceconn.ServiceConnection;
import gov.anl.dm.esafsync.serviceconn.StorageServiceConnection;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFormattedTextField.AbstractFormatter;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

import org.jdatepicker.impl.JDatePanelImpl;
import org.jdatepicker.impl.JDatePickerImpl;
import org.jdatepicker.impl.UtilDateModel;


class ExperimentList extends JFrame
{
	private static final long serialVersionUID = 1L;
	
	public class DateLabelFormatter extends AbstractFormatter {
		private static final long serialVersionUID = 1L;
		private String datePattern = "MM-dd-yyyy";
	    private SimpleDateFormat dateFormatter = new SimpleDateFormat(datePattern);

	    @Override
	    public Object stringToValue(String text) throws ParseException {
	        return dateFormatter.parseObject(text);
	    }

	    @Override
	    public String valueToString(Object value) throws ParseException {
	        if (value != null) {
	            Calendar cal = (Calendar) value;
	            return dateFormatter.format(cal.getTime());
	        }
	        return "";
	    }

	    public String valueToString(Date value) throws ParseException {
	        if (value != null) {
	            Date cal = (Date) value;
	            return dateFormatter.format(cal.getTime());
	        }
	        return "";
	    }

	}
	
	enum Column {
		NAME("Experiment name"),
		DESCRIPTION("Description"),
		START_DATE("Start Date"),
		END_DATE("End Date"),
		MANAGER("Manager(s)"),
		PI("Principal Investigator(s)"),
		USER("Users");

		private final String column;

		private Column(String column) {
			this.column = column;
		}

		@Override
		public String toString() {
			return column;
		}
	}

	enum Role {
		MANAGER("Manager"),
		PI("PI"),
		USER("User");
		
		private final String role;
		
		private Role(String role) {
			this.role = role;
		}

		@Override
		public String toString() {
			return role;
		}
	}
	
	class ExperimentTableModel extends AbstractTableModel {
		private static final long serialVersionUID = 1L;

		String dataValues [][];

		@Override
		public int getRowCount() {
			return dataValues.length;
		}

		@Override
		public int getColumnCount() {
			return Column.values().length;
		}

		@Override
		public Object getValueAt(int rowIndex, int columnIndex) {
			return dataValues[rowIndex][columnIndex];
		}

		public String getColumnName(int col) {
			return Column.values()[col].toString();
		}

		public Class getColumnClass(int c) {
			return getValueAt(0, c).getClass();
		}

		void setTable(String list) {
			List<String[]> expTable = new ArrayList<>();
			String [] rows = list.split("EXP");
			for (int i=0; i<rows.length; i++) {
				String [] cells = rows[i].split(",");
				boolean goodRow = false;
				int j = 0;
				for (j = 0; j < cells[0].length(); j++) {
					if ( Character.isDigit(cells[0].charAt(j)) ) {
						goodRow = true;
						break;
					}				
				}
				if (!goodRow) {
					continue;
				} else {
					// parse experiment data
					String[] experiment = new String[getColumnCount()+1];
					experiment[Column.NAME.ordinal()] = cells[0].substring(j);
					experiment[Column.DESCRIPTION.ordinal()] = cells[1].replace("$",",");
					experiment[Column.START_DATE.ordinal()] = cells[3];
					experiment[Column.END_DATE.ordinal()] = cells[4];
					experiment[Column.MANAGER.ordinal()] = parseUsers(cells[5]);
					experiment[Column.PI.ordinal()] = parseUsers(cells[6]);
					int lastIndex = cells[7].indexOf("\"");
					experiment[Column.USER.ordinal()] = parseUsers(cells[7].substring(0, lastIndex));
					expTable.add(experiment);
				}
			}
			dataValues = new String [expTable.size()][getColumnCount()];
			for (int i=0; i < expTable.size(); i++) {
				for (int j=0; j < getColumnCount(); j++) {
					dataValues[i][j] = expTable.get(i)[j];
				}
			}
		}

		private String parseUsers(String userList) {
			if (userList.equals("00000")) {
				return "";
			}
			String parsedUsers = "";
			String [] users = userList.split(":");
			for (int i=0; i < users.length; i++) {
				parsedUsers = parsedUsers + "d" + users[i] + " ";
			}
			return parsedUsers;
		}
	}

	public static final String EXPERIMENT_PREFIX = "esaf";
	StorageServiceConnection sconnection;
	private	JTable table = null;
	private ExperimentTableModel tableModel = null;
	private	JScrollPane scrollPane;

	ExperimentList(final String sector, final OracleConnection oconnection, final StorageServiceConnection sconnection, final DaqServiceConnection dconnection,  final DaqServiceConnection drconnection) {
		this.sconnection = sconnection;
		setTitle("Experiment Import"); 
		setSize(1000, 500);
		setBackground(Color.gray);

		final JPanel topPanel = new JPanel();
		topPanel.setBorder(BorderFactory.createLoweredSoftBevelBorder());
		topPanel.setLayout( new BorderLayout() );
		getContentPane().add( topPanel );

		JPanel entryPanel = new JPanel(new FlowLayout());
		topPanel.add(entryPanel, BorderLayout.NORTH);

		final JLabel entryLabel = new JLabel("Enter date range of experiment start date");
		entryPanel.add(entryLabel);

		UtilDateModel startModel = new UtilDateModel();
		Properties p = new Properties();
		p.put("text.today", "Today");
		p.put("text.month", "Month");
		p.put("text.year", "Year");
		JDatePanelImpl startDatePanel = new JDatePanelImpl(startModel, p);
		final JDatePickerImpl startDatePicker = new JDatePickerImpl(startDatePanel, new DateLabelFormatter());
		entryPanel.add(startDatePicker);

		UtilDateModel endModel = new UtilDateModel();
		JDatePanelImpl endDatePanel = new JDatePanelImpl(endModel, p);
		final JDatePickerImpl endDatePicker = new JDatePickerImpl(endDatePanel, new DateLabelFormatter());
		entryPanel.add(endDatePicker);

		final JButton submitDatesBtn = new JButton("click to get experiments list");
		entryPanel.add(submitDatesBtn);

		JPanel selectPanel = new JPanel(new FlowLayout());
		topPanel.add(selectPanel, BorderLayout.SOUTH);

		final JButton importExperimentBtn = new JButton("import selected experiments");
		selectPanel.add(importExperimentBtn);
		importExperimentBtn.setVisible(false);

		final JButton importAndStartExperimentBtn = new JButton("import and start selected experiments");
		selectPanel.add(importAndStartExperimentBtn);
		importAndStartExperimentBtn.setVisible(false);
		
		final JButton startExperimentBtn = new JButton("start selected experiments");
		selectPanel.add(startExperimentBtn);
		startExperimentBtn.setVisible(false);
		
		final JButton stopExperimentBtn = new JButton("stop selected experiments");
		selectPanel.add(stopExperimentBtn);
		stopExperimentBtn.setVisible(false);
		
		submitDatesBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				entryLabel.setVisible(true);
				submitDatesBtn.setVisible(true);
				importExperimentBtn.setVisible(true);
				importAndStartExperimentBtn.setVisible(true);
				stopExperimentBtn.setVisible(true);
				startExperimentBtn.setVisible(true);

				DateLabelFormatter df = new DateLabelFormatter();
				final Date startSelectedDate = (Date) startDatePicker.getModel().getValue();
				String start;
				String end;
				try {
					start = df.valueToString(startSelectedDate);
					Date endSelectedDate = (Date) endDatePicker.getModel().getValue();
					end = df.valueToString(endSelectedDate);
				} catch  (ParseException e1) {
					JOptionPane.showMessageDialog(null, e1.getMessage());
					return;
				}
				String list;
				try {
					list = oconnection.getExperiments(sector, start, end);
				}catch (Exception e1) {
					JOptionPane.showMessageDialog(null, e1.getMessage());						
					return;
				}
				if (table == null) {
					tableModel = new ExperimentTableModel();
					tableModel.setTable(list);
					table = new JTable(tableModel);
					scrollPane = new JScrollPane( table );
					topPanel.add( scrollPane, BorderLayout.CENTER );
					table.setRowSelectionAllowed( true );
					pack();
					setVisible(true);
					table.setVisible(true);
				} else {
					tableModel.setTable(list);
					tableModel.fireTableStructureChanged();
				}
			}			
		});

		importExperimentBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				int[] selection = table.getSelectedRows();
				for (int i = 0; i < selection.length; i++) {
					String experimentName = EXPERIMENT_PREFIX+(String)table.getModel().getValueAt(selection[i], Column.NAME.ordinal());
					String description = (String)table.getModel().getValueAt(selection[i], Column.DESCRIPTION.ordinal());
					String startDate = (String)table.getModel().getValueAt(selection[i], Column.START_DATE.ordinal());
					String endDate = (String)table.getModel().getValueAt(selection[i], Column.END_DATE.ordinal());
					if (sconnection.addExperiment(experimentName, description, startDate, endDate) != ServiceConnection.ServiceConnectionStatus.SUCCESS) {
						continue;
					}
					addRole((String)table.getModel().getValueAt(selection[i], Column.MANAGER.ordinal()), experimentName, Role.MANAGER);
					addRole((String)table.getModel().getValueAt(selection[i], Column.PI.ordinal()), experimentName, Role.PI);
					addRole((String)table.getModel().getValueAt(selection[i], Column.USER.ordinal()), experimentName, Role.USER);
					if (sconnection.startExperiment(experimentName) != ServiceConnection.ServiceConnectionStatus.SUCCESS) {
					  JOptionPane.showMessageDialog(null,"The experiment "+ experimentName +" did not start",
						"Info",JOptionPane.WARNING_MESSAGE);
					}
				}
				table.clearSelection();
			}			
		});

		importAndStartExperimentBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				int[] selection = table.getSelectedRows();
				if (selection.length > 0) {
					if (dconnection.isSessionValid() && ((drconnection == null) || drconnection.isSessionValid())) {
						importAndStartExperiments(selection, dconnection, drconnection);
					}  else {
						JOptionPane.showMessageDialog(null,"Session expired. Restart the tools.",
								"Info",JOptionPane.WARNING_MESSAGE);						
					}
				}
			}
		});

		startExperimentBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				int[] selection = table.getSelectedRows();
				if (selection.length > 0) {
					if (dconnection.isSessionValid() && ((drconnection == null) || drconnection.isSessionValid())) {
						startExperiments(selection, dconnection, drconnection);
					}  else {
						JOptionPane.showMessageDialog(null,"Session expired. Restart the tools.",
								"Info",JOptionPane.WARNING_MESSAGE);						
					}
				}
			}
		});

		stopExperimentBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				int[] selection = table.getSelectedRows();
				if (selection.length > 0) {
					if (dconnection.isSessionValid() && ((drconnection == null) || drconnection.isSessionValid())) {
						stopExperiments(selection, dconnection, drconnection);
					}  else {
						JOptionPane.showMessageDialog(null,"Session expired. Restart the tools.",
								"Info",JOptionPane.WARNING_MESSAGE);						
					}
				}
			}
		});

	}

	private void importAndStartExperiments(int[] selection, DaqServiceConnection dconnection, DaqServiceConnection drconnection) {
		for (int i = 0; i < selection.length; i++) {
			String experimentName = EXPERIMENT_PREFIX+(String)table.getModel().getValueAt(selection[i], Column.NAME.ordinal());
			String description = (String)table.getModel().getValueAt(selection[i], Column.DESCRIPTION.ordinal());
			String startDate = (String)table.getModel().getValueAt(selection[i], Column.START_DATE.ordinal());
			String endDate = (String)table.getModel().getValueAt(selection[i], Column.END_DATE.ordinal());
			if (sconnection.addExperiment(experimentName, description, startDate, endDate) != ServiceConnection.ServiceConnectionStatus.SUCCESS) {
				continue;
			}
			addRole((String)table.getModel().getValueAt(selection[i], Column.MANAGER.ordinal()), experimentName, Role.MANAGER);
			addRole((String)table.getModel().getValueAt(selection[i], Column.PI.ordinal()), experimentName, Role.PI);
			addRole((String)table.getModel().getValueAt(selection[i], Column.USER.ordinal()), experimentName, Role.USER);
			if (sconnection.startExperiment(experimentName) == ServiceConnection.ServiceConnectionStatus.SUCCESS) {
				// convert date
				String startDateDir;
				try {
					startDateDir = convertDate2Dir(startDate);
				} catch (ParseException e) {
					startDateDir = startDate;
				}
				if (drconnection != null) {
					drconnection.startDaq(experimentName, startDateDir);
				}
				dconnection.startDaq(experimentName, startDateDir);
			} else {
				JOptionPane.showMessageDialog(null,"The experiment "+ experimentName +" did not start",
						"Info",JOptionPane.WARNING_MESSAGE);
			}
		}
		table.clearSelection();		
	}
	
	private void startExperiments(int[] selection, DaqServiceConnection dconnection, DaqServiceConnection drconnection) {
		for (int i = 0; i < selection.length; i++) {
			String startDate = (String)table.getModel().getValueAt(selection[i], Column.START_DATE.ordinal());
			// convert date
			String startDateDir;
			try {
				startDateDir = convertDate2Dir(startDate);
			} catch (ParseException e) {
				startDateDir = startDate;
			}
			String experimentName = EXPERIMENT_PREFIX+(String)table.getModel().getValueAt(selection[i], Column.NAME.ordinal());
			if (drconnection != null) {
				drconnection.startDaq(experimentName, startDateDir);
			}
			dconnection.startDaq(experimentName, startDateDir);						
		}
		table.clearSelection();		
	}

	private void stopExperiments(int[] selection, DaqServiceConnection dconnection, DaqServiceConnection drconnection) {
		for (int i = 0; i < selection.length; i++) {
			String experimentName = EXPERIMENT_PREFIX+(String)table.getModel().getValueAt(selection[i], Column.NAME.ordinal());
			dconnection.stopDaq(experimentName);
			if (drconnection != null) {
				drconnection.stopDaq(experimentName);
			}
		}
		table.clearSelection();		
	}

	private void addRole(String userList, String experimentName, Role role) {
		if ((userList == null) || userList.isEmpty()) {
			return;
		}
		String [] users = userList.split(" ");
		for (int i = 0; i < users.length; i++) {
			sconnection.addExperimentUser(users[i], experimentName, role.toString());
		}
	}
	
	private String convertDate2Dir(String startDate) throws ParseException {
		String month = startDate.substring(3,6);
		String numMonth = null;
		if (month.equals("JAN")) {
			numMonth = "01";
		} else if (month.equals("FEB")) {
			numMonth = "02";
		} else if (month.equals("MAR")) {
			numMonth = "03";
		} else if (month.equals("APR")) {
			numMonth = "04";
		} else if (month.equals("MAY")) {
			numMonth = "05";
		} else if (month.equals("JUN")) {
			numMonth = "06";
		} else if (month.equals("JUL")) {
			numMonth = "07";
		} else if (month.equals("AUG")) {
			numMonth = "08";
		} else if (month.equals("SEP")) {
			numMonth = "09";
		} else if (month.equals("OCT")) {
			numMonth = "10";
		} else if (month.equals("NOV")) {
			numMonth = "11";
		} else if (month.equals("DEC")) {
			numMonth = "12";
		}
		
		return "20" + startDate.substring(7, 9) + "-" + numMonth;
	}

}


